﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Principal;
using System.Net.NetworkInformation;
using System.Net.Sockets;           // needed to find the network interface closest to the Server
using System.Net;
using Microsoft.Win32;

/************************************************************************
 * All the various internal Macros that will be interpreted by the launcher
 * shall be contained in this script.  The convention is to identify variables \ Macros
 * as a result of being encapsulated in percentage signs
 * 
 * 
 * **********************************************************************
 * */

namespace InstallXML
{
    partial class Program
    {

        private static string ReplaceVariables(string commandline)
        {

            System.OperatingSystem osInfo = System.Environment.OSVersion;
            //Console.WriteLine(osInfo.);


          
            // *********************
            //  USERDOMAIN
            //  Returns the domain of the logged on user
            // *********************
            
            WindowsIdentity id = WindowsIdentity.GetCurrent();
            string login = id.Name;  // login = "DOMAIN\Administrator" 
            string[] parts = login.Split('\\');
            string UserDomain = parts[0];
            commandline = commandline.Replace(@"@USERDOMAIN", UserDomain);


            // *********************
            //  SHORTDOMAIN
            //  Returns the domain of the logged on user
            // *********************

            string domainName = System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties().DomainName;
            commandline = commandline.Replace(@"@DOMAIN", domainName);

            string[] domaincomponents = domainName.Split('.');
            try
            {
                string shortdomain = domaincomponents[0] ;
                commandline = commandline.Replace(@"@SHORTDOMAIN", shortdomain);
            }
            catch
            {
            }



            // *********************
            // PATH
            // Returns the current directory path that contains the Launcher executable
            // *********************
            commandline = commandline.Replace(@"@PATH", path);

            // *********************
            // COMSPEC
            // *********************
            string Comspec = System.Environment.GetEnvironmentVariable("COMSPEC");
            // Console.WriteLine("Comspec =" + Comspec);
            commandline = commandline.Replace(@"@COMSPEC", Comspec);



            // *********************
            // PRODUCTNAME
            // *********************
            string ProductName = Convert.ToString(Registry.GetValue
                (@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion", "ProductName", ""));
            commandline = commandline.Replace(@"@PRODUCTNAME", ProductName);
            // Console.WriteLine("ProductName =" + ProductName);


            // *********************
            // @CPUArch
            // *********************
            string PROCESSOR_ARCHITECTURE = System.Environment.GetEnvironmentVariable("PROCESSOR_ARCHITECTURE");
             // Console.WriteLine("Windir =" + Windir);
            commandline = commandline.Replace(@"@CPUArch", PROCESSOR_ARCHITECTURE);
            // Console.WriteLine("CPUArch =" + PROCESSOR_ARCHITECTURE);



            // *********************
            // QUOTES ... not as simple as it first appears
            // 
            // Returnes Double, Single & Backslash Single Quotes
            // *********************

            //commandline = commandline.Replace(@"\\'", @"\'");
            commandline = commandline.Replace(@"\\'", @"^%#ESCQUOTE#%^"); 
            //commandline = commandline.Replace(@"\'", "'");
            commandline = commandline.Replace(@"\'", @"^%#SINGLEQUOTE#%^");

            // If it's not an escaped quote it may be changed normally to double quotes
            commandline = commandline.Replace(@"'", "\"");
            commandline = commandline.Replace(@"[" + (char)34 + "]", "\"");

            //Now the Single Quotes have been changed, escaped my be dealt with
            commandline = commandline.Replace(@"^%#ESCQUOTE#%^",@"\'"); //thanks to IIS Config
            commandline = commandline.Replace( @"^%#SINGLEQUOTE#%^",@"'");


            // *********************
            // PCNAME
            // Returns the computername
            // *********************
            commandline = commandline.Replace(@"@PCNAME", System.Environment.MachineName);
            // Console.WriteLine("PCNAME =" + System.Environment.MachineName);

            // *********************
            // COMPUTERNAME
            // Returns the full computername
            // *********************
            string computername = GetFQDN();
            commandline = commandline.Replace(@"@COMPUTERNAME", computername);
            // Console.WriteLine("COMPUTERNAME =" + computername);


            //Console.WriteLine("TCP start");

            // *********************
            // TCP/IP ADDRESS
            // Returns the publically facing TCP Address of the machine
            // *********************
            string IPAddressString = "";
           // if (commandline.Contains(@"@IPADDRESS") == true)
           // {
                IPAddress localAddr;
                
                try
                {
                    UdpClient u = new UdpClient(UserDomain, 1);
                    localAddr = ((IPEndPoint)u.Client.LocalEndPoint).Address;
                    commandline = commandline.Replace(@"@IPADDRESS", localAddr.ToString());
                     IPAddressString = localAddr.ToString();
                    // Console.WriteLine("IPADDRESS =" + localAddr.ToString());

                }
                catch
                {
                    IPAddressString = "";
                }
           // }

           // Console.WriteLine(IPAddressString);

            // *********************
            // WINDIR
            // *********************
            string Windir = System.Environment.GetEnvironmentVariable("WINDIR");
            commandline = commandline.Replace(@"@WINDIR", Windir);
            // Console.WriteLine("Windir =" + Windir);


            // *********************
            // SYSTEMDIR
            // *********************
            string SystemDir = System.Environment.SystemDirectory;
            commandline = commandline.Replace(@"@SYSTEMDIR", SystemDir);



            // *********************
            // TEMP
            // *********************
            string Temp = System.Environment.GetEnvironmentVariable("TEMP");
            commandline = commandline.Replace(@"@TEMP", Temp);
            // Console.WriteLine("Temp =" + Temp);
            // *********************
            // ALLUSERSPROFILE
            // *********************
            string ALLUSERSPROFILE = System.Environment.GetEnvironmentVariable("ALLUSERSPROFILE");
            commandline = commandline.Replace(@"@AllUsersProfile", ALLUSERSPROFILE);
            // Console.WriteLine("ALLUSERSPROFILE =" + ALLUSERSPROFILE);

            // *********************
            // APPDATA
            // *********************
            string APPDATA = System.Environment.GetEnvironmentVariable("APPDATA");
            commandline = commandline.Replace(@"@Appdata", APPDATA);
            // Console.WriteLine("APPDATA =" + APPDATA);

            // *********************
            // HOMEDRIVE
            // *********************
            string HOMEDRIVE = System.Environment.GetEnvironmentVariable("HOMEDRIVE");
            commandline = commandline.Replace(@"@Homedrive", HOMEDRIVE);
            // Console.WriteLine("HOMEDRIVE =" + HOMEDRIVE);

            // *********************
            // LOGONSERVER
            // *********************
            string LOGONSERVER = System.Environment.GetEnvironmentVariable("LOGONSERVER");
            commandline = commandline.Replace(@"@Logonserver", LOGONSERVER);
            // Console.WriteLine("LOGONSERVER =" + LOGONSERVER);

            //DET Specific 

            // *********************
            // SITENAME
            // *********************
            string SiteName = System.Environment.GetEnvironmentVariable("SITENAME");
          //  Console.WriteLine("SiteName =" + SiteName);
            commandline = commandline.Replace(@"@SITENAME", SiteName);
            // Console.WriteLine("SiteName =" + SiteName);
  
            return commandline;
        }





        static string determineOS()
        {
            // Get OperatingSystem information from the system namespace.
            System.OperatingSystem osInfo = System.Environment.OSVersion;

            // Determine the platform.
            switch (osInfo.Platform)
            {
                // Platform is Windows 95, Windows 98, 
                // Windows 98 Second Edition, or Windows Me.
                case System.PlatformID.Win32Windows:

                    switch (osInfo.Version.Minor)
                    {
                        case 0:
                            return "Windows 95";
                           // break;
                        case 10:
                            if (osInfo.Version.Revision.ToString() == "2222A")
                                //Console.WriteLine("Windows 98 Second Edition");
                                return "Windows 98 Second Edition";
                            else
                                return "Windows 98";
                                //Console.WriteLine("Windows 98");
                          //  break;
                        case 90:
                            return "Windows Me";
                           // Console.WriteLine("Windows Me");
                           // break;
                    }
                    break;

                // Platform is Windows NT 3.51, Windows NT 4.0, Windows 2000,
                // or Windows XP.
                case System.PlatformID.Win32NT:

                    switch (osInfo.Version.Major)
                    {
                        case 3:
                            return "Windows NT 3.51";
                            //break;
                        case 4:
                            return "Windows NT 4.0";
                            //break;
                        case 5:
                            if (osInfo.Version.Minor == 0)
                                return "Windows 2000";
                            else
                                return "Windows XP";
                            //break;
                        case 6:
                            return ReportWindowsMinorVersion(osInfo.Version.Minor);

                    } break;
            }
            return "";
        }

        public static string ReportWindowsMinorVersion(int Version)
        {
            if (Version == 0)
            {
                return "Windows Vista";
            }
            if (Version == 1)
            {
                return "Windows 7";
            }
            if (Version == 2)
            {
                return "Windows 8";
            }
            if (Version == 3)
            {
                return "Windows 8.1";
            }
            return "Unknown Version";
        }

        public static string GetFQDN()
        {
            string domainName = System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties().DomainName;
            string hostName = Dns.GetHostName();
            string fqdn = "";
            if (!hostName.Contains(domainName))
                fqdn = hostName + "." + domainName;
            else
                fqdn = hostName;

            return fqdn;
        }
    }
}
