﻿using System;
using System.Management;
using System.IO;

using System.Diagnostics; // used for running a program

using System.Reflection;
using System.Collections;
using System.Xml;
using System.Runtime.InteropServices;
using Microsoft.Win32; // Registry Reading
using System.Text;

using System.Security.Principal;
using System.Net.NetworkInformation;
using System.Net.Sockets;           // needed to find the network interface closest to the Server
using System.Net;
using Pahvant.MSI;


namespace InstallXML
{
    partial class Program
    {
        //declare variables
        static string path;
        static string PackageName ="";
        static string varMODE = "Install";
        //static string DistributionPoint ="";
        static int ReturnCode = 0;
        static string iName;
        private static string logFileName = "";


        public static int Main()
        {
            path = GetPath();
            GetStartupSwitches();
            CheckCreateLogDirectory();

            LogError("Launcher initialised", "INFORMATION", 3);

            int Returncode = ParseInstallXML();
           
            Environment.Exit(Returncode);
            return Returncode;
        }


        /// <summary>
        /// This routine will iteratively run through the various installation steps 
        /// within the accompanying Install.xml
        /// </summary>
        /// <returns></returns>
        public static int ParseInstallXML()
        {

            string Result = "FAIL";
            bool InstallFailed=false;
            bool ConditionsMet;

            XmlDocument xmlDoc = new XmlDocument();

            LogError("Loading InstallXML: " + path + (char)92 + "Install.xml", "INFORMATION", 3);

            if (File.Exists(path + (char)92 + "Install.xml"))
            {
                xmlDoc.Load(path + (char)92 + "Install.xml");
                XmlNode node = xmlDoc.DocumentElement;

                // Read the Productname from the XML - we will use this for logging

                string varPackageName = node.Attributes["Name"].Value;
                PackageName = varPackageName; //probably a duplicate - need to check later.

                logFileName = System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".log";


                // Create a list of the child nodes of 
                // the first node under the root
                XmlNodeList Actions = node.ChildNodes;

                // Visit each node
                for (int i = 0; i < Actions.Count; i++)
                {

                    if (Actions[i].Name.ToUpper() == varMODE.ToUpper())
                    {
                      //  Console.WriteLine("Mode = " + varMODE);
                        //LogError("Utility invoked in Launch Mode", "INFORMATION", 3);

                        XmlNodeList InstallActions = Actions[i].ChildNodes;
                        // Display the value of its first child node
                        for (int j = 0; j < InstallActions.Count; j++)
                            if (InstallFailed == false)
                            {
                                switch (InstallActions[j].Name)
                                {
                                    case "Installer":

                                        string iName = @InstallActions[j].Attributes["Name"].Value;
                                        string iType = @InstallActions[j].Attributes["Type"].Value;
                                        string iTarget = @InstallActions[j].Attributes["Target"].Value;
                                        string iArgs = @InstallActions[j].Attributes["Args"].Value;
                                        string iArgs2 = @InstallActions[j].Attributes["Args2"].Value;
                                        string iEnc = @InstallActions[j].Attributes["Enc"].Value;
                                        string Conditions = @InstallActions[j].Attributes["Conditions"].Value;

                                        ConditionsMet = CheckConditions(Conditions);
                                        if ((ConditionsMet == true) && (InstallFailed == false))
                                        {
                                            LogError("Processing Install Type " + iType, "INFORMATION", 3);
                                            switch (iType)
                                            {
                                                case "MSIRM":
                                                    Console.WriteLine("MSIRM");
                                                    Result = RunMSIRemoval(iTarget, varPackageName, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                    }
                                                    else
                                                    {
                                                        InstallFailed = true;
                                                    }
                                                    break;


                                                case "EXE":
                                                    Console.WriteLine("EXE");

                                                    //Verify that the Target Executable has a full path
                                                    //Check if the file exists in a directory
                                                    //or outside a directory at an absolute path

                                                    if ((confirmFileExists(path + (char)92 + iTarget) == true) || (confirmFileExists(ReplaceVariables(iTarget)) == true))
                                                    {
                                                        if (confirmFileExists(path + (char)92 + iTarget) == true)
                                                        {
                                                            iTarget = path + (char)92 + iTarget;
                                                        }

                                                        if (confirmFileExists(ReplaceVariables(iTarget)) == true)
                                                        {
                                                            iTarget = ReplaceVariables(iTarget);
                                                        }

                                                        Result = RunExe(iTarget, ReplaceVariables(iArgs), varPackageName, iArgs2, iName);
                                                        if (Result == "SUCCESS")
                                                        {
                                                            LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        }
                                                        else
                                                        {
                                                            LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                            InstallFailed = true;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        // It may be that the file exists in another location


                                                        LogError(iName + " Install Failed" + " File Not Found " + iTarget, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;


                                                case "APPV":
                                                    Console.WriteLine("APPV");
                                                    Result = RunAppV(iTarget, iArgs, varPackageName, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;
                                                case "APPVRM":
                                                    Console.WriteLine("APPVRM");
                                                    Result = RunAppVRM(varPackageName, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;
                                                case "MSI":
                                                    Console.WriteLine("MSI");

                                                    if (confirmFileExists(path + (char)92 + iTarget) == true)
                                                    {
                                                        Result = RunMSI(path + (char)92 + iTarget, iArgs, iArgs2, varPackageName, iName);
                                                        if (Result == "SUCCESS")
                                                        {
                                                             LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        }
                                                        else
                                                        {
                                                            LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                            InstallFailed = true;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " File Not Found " + iTarget, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;


                                                case "MSU":
                                                    Console.WriteLine("MSU");
                                                    Result = RunMSU(path + (char)92 + iTarget, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;
                                                case "MSP":
                                                    Console.WriteLine("MSP");
                                                    Result = RunMSP(path + (char)92 + iTarget, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                      }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;
                                                case "CMD":
                                                    Console.WriteLine("CMD");
                                                    Result = RunCMD(iTarget, iArgs, varPackageName, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;
                                                case "PS1":
                                                    Console.WriteLine("PS1");
                                                    Result = RunPS1(iTarget, iArgs, varPackageName, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        //    return (ReturnCode);
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;

                                                case "VBS":
                                                    Console.WriteLine("VBS");
                                                    Result = RunVBS(iTarget, iArgs, iArgs2, varPackageName, iName);
                                                    if (Result == "SUCCESS")
                                                    {
                                                        LogError(iName + " Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        // return (ReturnCode);
                                                    }
                                                    else
                                                    {
                                                        LogError(iName + " Install Failed" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                                                        InstallFailed = true;
                                                    }
                                                    break;
                                                default:
                                                    break;
                                            }
                                        }

                                        break;

                                }

                            } //Installfailed = false
                            else
                            {
                                LogError(iName + " Processing ignored due to previous item failure ", "INFORMATION", 1);
                            }

                        if (InstallFailed == false) //Nothing in the set failed
                        {
                            //Create Signature as the Install sections have not created a failed install
                            System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log");
                            System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSU.log");
                            System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".log");
                            System.IO.File.WriteAllText(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".sig", Environment.UserName + " " + DateTime.Now);

                        }
                        else
                        {
                            LogError(iName + " Package Install Failed" + " Return Code = " + ReturnCode, "ERROR", 1);
                            return ReturnCode;
                           }
                    }

                }
            }
            else
            {   // The Install.xml file cannot be found
                LogError((char)34 + path + (char)92 + "Install.xml" + (char)34 + " does not exist", "ERROR", 1);
            }

            if (ReturnCode == 3010)
            {
                return 3010;
            }
            else
            {
                return 0;
            }
        }

        /// <summary>
        /// Purpose is to check files exist before calling them
        /// </summary>
        /// <param name="varFilename"></param>
        /// <returns>true / false</returns>
        private static bool confirmFileExists(string varFilename)
        {
            if (varFilename.Length == 0)
            {
                return true;
            }
            else
            {
                if (File.Exists(varFilename))
                {
                    // Console.WriteLine("The file exists. " + varFilename);
                    return true;
                }
                else
                {
                    Console.WriteLine("File not Found: " + varFilename);
                    return false;
                }
            }
              
          
        }

        /// <summary>
        /// Used to create an example template for use with the Launcher
        /// </summary>
        public static void CreateBlankTemplate(){

            Console.WriteLine("Blank Template Path = " + path);
        //    Using (System.IO.StreamWriter file = new System.IO.StreamWriter(path +@"\Install.xml"))
        //    {

            TextWriter tsw = new StreamWriter(path + @"\Install.xml");

                tsw.WriteLine(@"<?xml version=" + (char)34 +"1.0" + (char)34 +" encoding=" + (char)34 +"utf-8" + (char)34 +" standalone=" + (char)34 +"yes" + (char)34 +"?>");
                tsw.WriteLine("");
                tsw.WriteLine(@"<!-- ###############"); 
                tsw.WriteLine(@"     Launcher");
                tsw.WriteLine(@"     Created by Laurie Rhodes");
                tsw.WriteLine(@"     GPL 3.0 License ");
                tsw.WriteLine(@"     for detailed help: Launcher.exe /?");
                tsw.WriteLine(@"     ############### -->"); 
                tsw.WriteLine("");
                tsw.WriteLine(@"<!-- The formal Package name  -->");
                tsw.WriteLine("");
                tsw.WriteLine(@"<Package Name=" + (char)34 + "My_PackageName" + (char)34 + ">");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <!-- Properties are used for SCCM Application creation -->");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <Properties>");
                tsw.WriteLine(@"    <Description>eg. Utility used for ... </Description>");
                tsw.WriteLine(@"    <Owner>eg. username@domain.com</Owner>");
                tsw.WriteLine(@"    <Icon></Icon>");
                tsw.WriteLine(@"    <Publisher>Manufacturer</Publisher>");
                tsw.WriteLine(@"    <Version>1.0</Version>");
                tsw.WriteLine(@"    <Notes>Notes to Deployment Staff members</Notes>");
                tsw.WriteLine(@"    <AppID>Windows8.StartTile.AppID</AppID>");
                tsw.WriteLine(@"  </Properties>");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <!-- Dependencies must be listed in order -->");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <Dependencies>");
                tsw.WriteLine(@"     <!-- <PreReq>PackageName_09.00_64a</PreReq> -->");
                tsw.WriteLine(@"  </Dependencies>");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <!-- Install Tasks (In Order) -->");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <Install>");
                tsw.WriteLine(@"    <!-- <Installer Name=" + (char)34 + "MSIInstall" + (char)34 + " Type=" + (char)34 + "MSI" + (char)34 + " Target=" + (char)34 + "Setup.Msi" + (char)34 + " Args=" + (char)34 + "eg TRANSFORM.mst" + (char)34 + " Args2=" + (char)34 + (char)34 + " Enc=" + (char)34 + "False" + (char)34 + " Conditions=" + (char)34 + (char)34 + @" /> -->");
                tsw.WriteLine(@"    <!-- <Installer Name=" + (char)34 + "Patch" + (char)34 + " Type=" + (char)34 + "MSP" + (char)34 + " Target=" + (char)34 + "MyPatch.MSP" + (char)34 + " Args=" + (char)34 + @"" + (char)34 + " Args2=" + (char)34 + "" + (char)34 + " Enc=" + (char)34 + "False" + (char)34 + " Conditions=" + (char)34 + (char)34 + @" --> />");
                tsw.WriteLine(@"    <!-- <Installer Name=" + (char)34 + "EXEInstall" + (char)34 + " Type=" + (char)34 + "EXE" + (char)34 + " Target=" + (char)34 + "Setup.EXE" + (char)34 + " Args=" + (char)34 + @"eg /ARGS" + (char)34 + " Args2=" + (char)34 + "ReturnCode" + (char)34 + " Enc=" + (char)34 + "False" + (char)34 + " Conditions=" + (char)34 + (char)34 + @" /> -->");
                tsw.WriteLine(@"    <!-- <Installer Name=" + (char)34 + "Powershell" + (char)34 + " Type=" + (char)34 + "PS1" + (char)34 + " Target=" + (char)34 + "Installer.ps1" + (char)34 + " Args=" + (char)34 + @"eg -ExecutionPolicy Unrestricted" + (char)34 + " Args2=" + (char)34 + "ReturnCode" + (char)34 + " Enc=" + (char)34 + "False" + (char)34 + " Conditions=" + (char)34 + (char)34 + @" /> -->");

                tsw.WriteLine(@"  </Install>");
                tsw.WriteLine(@"");
                tsw.WriteLine(@"  <!-- Removal Tasks (In Order) -->");
                tsw.WriteLine("");
                tsw.WriteLine(@"  <Remove>");
                tsw.WriteLine(@"    <!--    <Installer Name=" + (char)34 + "MSIRemove" + (char)34 + " Type=" + (char)34 + "MSIRM" + (char)34 + " Target=" + (char)34 + "{PRODUCT-CODE-0000-0000-85C245210F94}" + (char)34 + " Args=" + (char)34 + (char)34 + " Args2=" + (char)34 + (char)34 + " Enc=" + (char)34 + "False" + (char)34 + " Conditions=" + (char)34 + (char)34 + @" /> -->");
                tsw.WriteLine(@"  </Remove>");
                tsw.WriteLine(@"");
                tsw.WriteLine(@"</Package>");
                tsw.WriteLine(@"");
      //      }

                //Close the file.
                tsw.Close();


        }

        public static void Log(String logMessage, TextWriter writer)
        {
            writer.WriteLine(logMessage);
            writer.Flush();
        }
        public static void DumpLog(StreamReader reader)
        {
            // While not at the end of the file, read and write lines.
            String line;
            while ((line = reader.ReadLine()) != null)
            {
                Console.WriteLine(line);
            }
            reader.Close();
        }





        /// <summary>
        /// Check to Ensure Log directory Exists
        /// </summary>
        public static void CheckCreateLogDirectory()
        {
            if (!Directory.Exists(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\"))
            {
                Directory.CreateDirectory(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\");
            }
        }




        /// <summary>
        /// Returns the current path of the directory
        /// </summary>
        /// <returns></returns>
        public static string GetPath()
        {
            path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).Replace(@"file:\", "");
            if (path.IndexOf(":\\") != 1)
            {

                //supposedly this is a UNC Path
                Console.WriteLine("UNC Path");
                Console.WriteLine("Index Of=" + path.IndexOf(":\\"));
                path = @"\\" + path;
                return path.ToLower();
            }
            return path.ToLower();
        }

        /// <summary>
        /// Reads the Product Code from an existing MSI file.
        /// </summary>
        /// <param name="varMSIFile"></param>
        /// <returns></returns>
        public static string MSIGetProductCode(string varMSIFile)
        {
        Pahvant.MSI.Database database = new Pahvant.MSI.Database( varMSIFile, Pahvant.MSI.Database.OpenMode.ReadOnly );
        Pahvant.MSI.View view = new Pahvant.MSI.View( database, "select `Value` from `Property` where `Property`='ProductCode'");
        view.Execute();
        Record record = view.Fetch();
        string productCode = record.GetString(1);
        return productCode;
        }

        /// <summary>
        /// Retrieves Version of product from within a Windows Installer package
        /// </summary>
        /// <param name="varMSIFile"></param>
        /// <returns></returns>
        public static string MSIGetProductVersion(string varMSIFile)
        {
           // Console.WriteLine("MSIGetProductVersion " + varMSIFile);
            Pahvant.MSI.Database database = new Pahvant.MSI.Database(varMSIFile, Pahvant.MSI.Database.OpenMode.ReadOnly);
            Pahvant.MSI.View view = new Pahvant.MSI.View(database, "select `Value` from `Property` where `Property`='ProductVersion'");
            view.Execute();
            Record record = view.Fetch();
            string productVersion = record.GetString(1);
            return productVersion;
        }    





        /// <summary>
        ///   Creates a local Status MIF to return status of n install
        /// </summary>
        /// <param name="varPackageName">The Package that has been run</param>
        /// <param name="varStatus">The Success or failure of the process</param>
        /// <param name="varResult">The Result in Text Form or what has happened</param>
        /// <param name="iName">Used to identify which item failed in the package</param>
        public static void CreateStatusMIF(string varPackageName, string varStatus, string varResult, string iName)
        {
            StreamWriter sw = null;
            // all MIF files are located under with Windows folder.
            // the target directory is not negotiable 

            //Delete any pre-existing file 
            if (File.Exists(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\" + varPackageName + ".mif")) 
            {
                File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\" + varPackageName + ".mif");
            }

            // Open a new stream for the file
            FileStream fs = File.Open(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\" + varPackageName + ".mif", FileMode.CreateNew, FileAccess.Write);


            LogError(varPackageName + " status = " + varStatus + " Result = " + varResult, "INFORMATION", 3);



            // generate a file stream with UTF8 characters          
            sw = new StreamWriter(fs, System.Text.Encoding.UTF8);

            sw.WriteLine("START COMPONENT");
            sw.WriteLine("NAME = " + (char)34 + "WORKSTATION" + (char)34); 
            sw.WriteLine("  START GROUP"); 
            sw.WriteLine("NAME = " + (char)34 + "ComponentID" + (char)34); 
            sw.WriteLine("    ID = 1"); 
            sw.WriteLine("    CLASS = "  + (char)34 + "DMTF|ComponentID|1.0"  + (char)34); 
            sw.WriteLine("    START ATTRIBUTE"); 
            sw.WriteLine("      NAME = "  + (char)34 + "Product"  + (char)34); 
            sw.WriteLine("      ID = 2"); 
            sw.WriteLine("      ACCESS = READ-ONLY"); 
            sw.WriteLine("      STORAGE = SPECIFIC"); 
            sw.WriteLine("      TYPE = STRING(64)");
            sw.WriteLine("      VALUE = " + (char)34 + varPackageName + (char)34); 
            sw.WriteLine("    END ATTRIBUTE"); 
            sw.WriteLine("    START ATTRIBUTE"); 
            sw.WriteLine("      NAME = "  + (char)34 + "Installation"  + (char)34); 
            sw.WriteLine("      ID = 6"); 
            sw.WriteLine("      ACCESS = READ-ONLY"); 
            sw.WriteLine("      STORAGE = SPECIFIC"); 
            sw.WriteLine("      TYPE = STRING(64)");
            sw.WriteLine("      VALUE = " + (char)34 + String.Format("{0:G}", DateTime.Now) + (char)34); 
            sw.WriteLine("    END ATTRIBUTE"); 
            sw.WriteLine("  END GROUP"); 
            sw.WriteLine("  START GROUP"); 
            sw.WriteLine("    NAME = "  + (char)34 + "InstallStatus"  + (char)34); 
            sw.WriteLine("    ID = 2"); 
            sw.WriteLine("    CLASS = "  + (char)34 + "MICROSOFT|JOBSTATUS|1.0"  + (char)34); 
            sw.WriteLine("    START ATTRIBUTE"); 
            sw.WriteLine("      NAME = "  + (char)34 + "Status"  + (char)34); 
            sw.WriteLine("      ID = 1"); 
            sw.WriteLine("      ACCESS = READ-ONLY"); 
            sw.WriteLine("      STORAGE = SPECIFIC"); 
            sw.WriteLine("      TYPE = STRING(32)");
            sw.WriteLine("      VALUE = " + (char)34 + varStatus + (char)34); 
            sw.WriteLine("    END ATTRIBUTE"); 
            sw.WriteLine("    START ATTRIBUTE"); 
            sw.WriteLine("      NAME = "  + (char)34 + "Description"  + (char)34); 
            sw.WriteLine("      ID = 2"); 
            sw.WriteLine("      ACCESS = READ-ONLY"); 
            sw.WriteLine("      STORAGE = SPECIFIC"); 
            sw.WriteLine("      TYPE = STRING(128)");
            sw.WriteLine("      VALUE = " + (char)34 + varResult + " - process: " + iName + (char)34); 
            sw.WriteLine("    END ATTRIBUTE"); 
            sw.WriteLine("  END GROUP"); 
            sw.WriteLine("END COMPONENT"); 
            sw.WriteLine("");
            sw.Close();
        }

        public static void LogError(string varEvent, string varType, int ErrorCode)
        {
            // Create the Source if it doesnt exist
            if (!EventLog.SourceExists("Launcher"))
            {
                EventLog.CreateEventSource("Launcher", "Application");
            }

            Console.WriteLine(varEvent);
            // Write to the physical log as well
            if (logFileName != "")
            {
                using (StreamWriter writer = File.AppendText(logFileName))
                {
                    DateTime now = DateTime.Now;
                 

                    switch (varType)
                    {
                        case "ERROR":
                            EventLog.WriteEntry("Launcher", varEvent, EventLogEntryType.Error, ErrorCode);
                            Log(TraceFormat(now + " ***   " + iName + varEvent + " Return Code = " + ReturnCode, "3", "0", "Launcher.exe"), writer);
                            break;
                        case "WARNING":
                            EventLog.WriteEntry("Launcher", varEvent, EventLogEntryType.Warning, ErrorCode);
                            Log(TraceFormat(now + " ***   " +iName + varEvent + " Return Code = " + ReturnCode, "2", "0", "Launcher.exe"), writer);
                            break;
                        default: //Unless stated all logs will be informational
                            EventLog.WriteEntry("Launcher", varEvent, EventLogEntryType.Information, ErrorCode);
                            Log(TraceFormat(now + " ***   " + iName + varEvent, "1", "0", "Launcher.exe"), writer);
                            break;
                    }


                }
            }
        }
        
        /// <summary>
        /// Routine to check if the conditions of an install item are true.
        /// </summary>
        /// <param name="Conditions"></param>
        /// <returns></returns>
        public static bool CheckConditions(string Conditions)
        {
            string[] logicarray;

            

            if (Conditions == "")
            {
                LogError("No Conditional Logic Exists ", "INFORMATION", 3);
                return true;
            }
            else
            {
                Console.WriteLine(Conditions);
                // Start the Logic behind the conditions - replace variables with full strings
                Conditions = ReplaceVariables(@Conditions);

                if (Conditions.Contains(@"[ARP]=") == true)
                {   
                    logicarray = Conditions.Split('=');
                    string DesiredApp = logicarray[1].Replace("'", "");
                    DesiredApp = DesiredApp.Trim();

                    Console.WriteLine("DesiredApp = '" + DesiredApp + "'");
                    string TestString ="";

                    string registry_key = @"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall";
                    using (Microsoft.Win32.RegistryKey key = Registry.LocalMachine.OpenSubKey(registry_key))
                    {
                        foreach (string subkey_name in key.GetSubKeyNames())
                        {
                            try
                            {

                                using (RegistryKey subkey = key.OpenSubKey(subkey_name))
                                {
                                    TestString = subkey.GetValue("DisplayName").ToString();
                                    Console.WriteLine(" '" + TestString + "'");


                                    if (DesiredApp.ToUpper() == TestString.ToUpper())
                                    {
                                        LogError("Positive Logic Conditions = " + @DesiredApp + " Is Installed", "INFORMATION", 3);
                                        return true;
                                    }
                                }
                            }
                            catch
                            {
                            }
                        }

                    }

                    //64 bit reflection keys
                    registry_key = @"SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall";
                    using (Microsoft.Win32.RegistryKey key = Registry.LocalMachine.OpenSubKey(registry_key))
                    {
                        try
                        {

                            foreach (string subkey_name in key.GetSubKeyNames())
                            {
                                try
                                {

                                    using (RegistryKey subkey = key.OpenSubKey(subkey_name))
                                    {
                                        TestString = subkey.GetValue("DisplayName").ToString();

                                        if (DesiredApp.ToUpper() == TestString.ToUpper())
                                        {
                                            LogError("Positive Logic Conditions = " + @DesiredApp + " Is Installed", "INFORMATION", 3);
                                            return true;
                                        }
                                    }
                                }
                                catch
                                {
                                }
                            }
                        }
                        catch
                        {
                        }
                    }
                    return false;
               }


                if (Conditions.Contains(@"[ARP]<>") == true)
                {
                    logicarray = Conditions.Split('>');
                    string TestString = "";
                    string DesiredApp = logicarray[1].Replace("'", "");
                    DesiredApp = DesiredApp.Trim();

                    Console.WriteLine("DesiredApp = '" + DesiredApp + "'");
                    // Console.WriteLine("Desired App = " +DesiredApp);

                    string registry_key = @"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall";
                    using (Microsoft.Win32.RegistryKey key = Registry.LocalMachine.OpenSubKey(registry_key))
                    {
                        foreach (string subkey_name in key.GetSubKeyNames())
                        {
                            //  Console.WriteLine("Subkey = " + subkey_name);
                            try
                            {

                                using (RegistryKey subkey = key.OpenSubKey(subkey_name))
                                {
                                    TestString = subkey.GetValue("DisplayName").ToString();
                                    Console.WriteLine(" '" + TestString + "'");


                                    if (DesiredApp.ToUpper() == TestString.ToUpper())
                                    {
                                        LogError("Positive Logic Conditions = " + @DesiredApp + " Is Installed", "INFORMATION", 3);
                                        return false; // condition is that the application does not exist
                                    }
                                }
                            }
                            catch
                            {
                            }
                        }

                    }

                    //64 bit reflection keys
                    //Console.WriteLine("********************************");
                    //Console.WriteLine("64 bit registry queries");
                    //Console.WriteLine("********************************");
                    registry_key = @"SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall";
                    using (Microsoft.Win32.RegistryKey key = Registry.LocalMachine.OpenSubKey(registry_key))
                    {
                        try
                        {

                            foreach (string subkey_name in key.GetSubKeyNames())
                            {
                                try
                                {

                                    using (RegistryKey subkey = key.OpenSubKey(subkey_name))
                                    {
                                        TestString = subkey.GetValue("DisplayName").ToString();

                                        if (DesiredApp.ToUpper() == TestString.ToUpper())
                                        {
                                            LogError("Positive Logic Conditions = " + @DesiredApp + " Is Installed", "INFORMATION", 3);
                                            return false; // condition is that the application does not exist
                                        }
                                    }
                                }
                                catch
                                {
                                }
                            }
                        }
                        catch
                        {
                        }
                    }
                    return true;
                }

                if (Conditions.Contains(@"<>") == true)
                {   //split finction only accepts a single character
                    Conditions.Replace("<>", "=");
                    logicarray = Conditions.Split('=');
                    if (logicarray[0] != logicarray[1])
                    {
                        LogError("Positive Logic Conditions = " + @logicarray[0], "INFORMATION", 3);
                        return true;
                    }
                    else
                    {
                        LogError("Negative Logic Conditions = " + @logicarray[0], "INFORMATION", 3);
                        return false;
                    }
                }
                else
                    if (Conditions.Contains("=") == true)
                    {
                        logicarray = Conditions.Split('=');
                        if (logicarray[0] == logicarray[1])
                        {
                            LogError("Positive Logic Conditions = " + @logicarray[0], "INFORMATION", 3);
                            return true;
                        }
                        else
                        {
                            LogError("Negative Logic Conditions = " + @logicarray[0], "INFORMATION", 3);
                            return false;
                        }

                    }
                //By default, everything should be true
                return true;
            }

        }

        
        /// <summary>
        /// Ensures that file rename operations aren't pending that could prevent a successful install
        /// </summary>
        /// <returns></returns>
        public static bool CheckFileRenamePending()
        {
           // Console.WriteLine("Checking Registry");
            RegistryKey openKey = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\Control\Session Manager");

            // The Pending file rename value may not even exist
            object o = openKey.GetValue("PendingFileRenameOperations");
            string stringValue = o as string;



            if (stringValue != null)
            {
                // it's a string
            }
            else
            {
                string[] stringArray = o as string[];
                if (stringArray != null)
                {
                      if (stringArray.Length == 0) // make sure that the array isnt empty
                    {
                        return false;
                    }

                    return true;
                }
            }


            return false;

        }


        /// <summary>
        /// Returns the full path of the file to be called.
        /// By default, local to the current directory is assumed unless
        /// :\ are found at the start of the passed string
        /// </summary>
        /// <param name="varEXEfile"></param>
        /// <returns></returns>
        public static string CheckReturnFullPath(string varEXEfile)
        {
            string ExeFullFath = "";
            path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).Replace(@"file:\", "");
            if (path.IndexOf(":\\") != 1)
            {
                 path = @"\\" + path;
            }
            path = path.ToLower();

            //Parse the install exe path for variables such as comspec, windir etc
            varEXEfile = ReplaceVariables(varEXEfile);

            //We normally expect the executable to reside within the package folder
            //There are instances when executables must be run from ouside of the package directory
            //System32 is a good example.  If the executable is referenced from a drive mapping we wont
            //append the current path to the executable file

            if (@varEXEfile.IndexOf(":\\") == 1)
            {
                @ExeFullFath = @varEXEfile;
            }
            else
            {
               @ExeFullFath = path + (char)92 + @varEXEfile;
            }
            return ExeFullFath;
        }


        /// <summary>
        /// Routine converts registry versions to decimal
        /// </summary>
        /// <param name="stringversion"></param>
        /// <returns></returns>
        public static int[] versionfromreg(string stringversion)
        {
            int intversion = Convert.ToInt32(stringversion);
            //5001B9D   -  convert to Hex
            //83893149   - Registry entry in Decimal 
            string strversion = intversion.ToString("X");

            //05001B9D - Pad from left with "0" characters to be 8 characters in length
            //The only possiblities are 7 or 8 characters

            if (strversion.Length == 7)
            {
                strversion = "0" + strversion;
            }

           // Console.WriteLine("strversion  +" + strversion);
            //version lengeth should be 8 characters in length            
            //05 00 1B9D - separate to Major & Minor versions with Revision
            string version = "" 
                +int.Parse(strversion.Substring(0, 2), System.Globalization.NumberStyles.HexNumber) +"." +
                 int.Parse(strversion.Substring(2, 2), System.Globalization.NumberStyles.HexNumber) +"." +
                 int.Parse(strversion.Substring(4, 4), System.Globalization.NumberStyles.HexNumber); 

            //05 00 7069 - Convert Revision from Hex to Decimal
            int[] VersionNumbers = { int.Parse(strversion.Substring(0, 2), System.Globalization.NumberStyles.HexNumber), 
                                     int.Parse(strversion.Substring(2, 2), System.Globalization.NumberStyles.HexNumber), 
                                     int.Parse(strversion.Substring(4, 4), System.Globalization.NumberStyles.HexNumber) };
            //return version;
            return VersionNumbers;


        }



            public static string BaseDecode(string packageguid){
                //ensure 
                packageguid = packageguid.Replace(" ", "");
                packageguid = packageguid.Replace("-", "");

                string[] segments = new string[4] { packageguid.Substring(1, 5), packageguid.Substring(6, 5), packageguid.Substring(11, 5), packageguid.Substring(16, 5) };
                foreach (string s in segments)
                {
                    Console.WriteLine(" " +s);
                }

                return null;
            }

        /// <summary>
        /// Returns the Packed GUID for a Product Code
        /// </summary>
        /// <param name="packageguid"></param>
        /// <returns></returns>
            public static string packguid(string packageguid)
            {
                string packedguid="";

                //the string should be in upper anyway
                packageguid = packageguid.ToUpper();
                packageguid = packageguid.Replace(" ", "");
                packageguid = packageguid.Replace("-", "");

                packedguid = StringHelper.ReverseString(packageguid.Substring(1, 8));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(9, 4));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(13, 4));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(17, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(19, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(21, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(23, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(25, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(27, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(29, 2));
                packedguid = packedguid + StringHelper.ReverseString(packageguid.Substring(31, 2));

                return packedguid;
            }

        /// <summary>
        /// Writes and SCCM 2012 Distribution Point
        /// Accepts packged GUID
        /// Determines SCCM DP for CAS.log then parsing LocationServices.LOG
        /// </summary>
        /// <param name="PackedGuid"></param>
        public static void CheckCreateSCCM2012RepairPoint(string PackedGuid){

            string SCCMCache = "C:\\Windows\\CCMCache";
            string ContentID = "";
            string SCCMDP = "";

            path = path.ToLower();

            if (path.IndexOf(SCCMCache.ToLower()) >= 0)
            {

                string line;

                Console.WriteLine("SCCM Cache Install");

                  try
                {
                     FileStream CASfs = new FileStream("C:\\Windows\\CCM\\Logs\\CAS.log", FileMode.Open, FileAccess.Read, System.IO.FileShare.ReadWrite);
                    //string readText = File.ReadAllText("C:\\Windows\\CCM\\Logs\\CAS.log", Encoding.UTF8);
                    Console.WriteLine("CAS Log parsed");
                    using (var textReader = new StreamReader(CASfs))
                    //using (StringReader reader = new StringReader(readText))
                    {
                        Console.WriteLine("Using CAS stream");
                        //LogError("Stream Reading CAS.log", "INFORMATION", 1);
                        // var content = textReader.ReadLine(); 
                        // if (CASfs.CanRead)
                        
                        //{
                            // Read the file and display it line by line.
                            //LogError("StreamReader Handle Created", "INFORMATION", 3);
                            try
                            {

                                while ((line = textReader.ReadLine()) != null)
                                //while ((line = reader.ReadLine()) != null)
                                {
                                    line = line.ToLower();
                                    if (line.IndexOf(", " + path.ToLower()) >= 0) // '&& line.IndexOf(path.ToLower()
                                    {
                                        if (line.IndexOf("saved content id mapping") >= 0) // '&& line.IndexOf(path.ToLower()
                                        {
                                            // This line links the downloaded cache with the SCCM Content ID
                                            // The string always begins 32 characters into the line

                                            ContentID = line.Substring(32, (line.IndexOf(",") - 32));
                                            
                                        }
                                    }
                                }
                            }
                            catch (Exception e)
                            { // if anything has gone wrong with the string it cant be fixed from here
                                LogError("Fatal Error attempting to read CAS.log: " + e.ToString(), "ERROR", 1);
                            }
                            finally
                            {
                                //reader.Close();
                                textReader.Close();

                            }
                        
                       // }
                       // else
                       // {
                       //     LogError("C:\\Windows\\CCM\\Logs\\CAS.log" + " Unreadable", "ERROR", 1);
                       //     ;
                       // }
                    }

                    Console.WriteLine("ContentID = " + ContentID);
                }
                catch (IOException e)
                {
                    Console.WriteLine("Exception Caught");
                    LogError("IO caught " + e.ToString(), "INFORMATION", 1);
                    int pauseTime = 5000;
                    System.Threading.Thread.Sleep(pauseTime);
                }
                finally
                {
                   
                }




                // If a valid SCCM 2012 Content ID has been located
                // This needs to be searched for in the Location Services log
                // The 
                if (ContentID.Length >= 0)
                {
                    try
                    {
                        LogError("Opening Location Services Log", "INFORMATION", 1);
                        //string LocationServicesText = File.ReadAllText("C:\\Windows\\CCM\\Logs\\LocationServices.log", Encoding.UTF8);

                        FileStream LocationServicesfs = new FileStream("C:\\Windows\\CCM\\Logs\\LocationServices.log", FileMode.Open, FileAccess.Read, System.IO.FileShare.ReadWrite);
                        //using (StringReader reader = new StringReader(LocationServicesText))
                        using (var textReader = new StreamReader(LocationServicesfs))
                        {
                            // var content = textReader.ReadLine(); ;
                            //if (LocationServicesfs.CanRead)
                            //{
                                // Read the file and display it line by line.
                            try
                            {

                                // while ((line = reader.ReadLine()) != null)
                                while ((line = textReader.ReadLine()) != null)
                                {
                                    line = line.ToLower();
                                    if (line.IndexOf("distribution point") >= 0)
                                    {

                                        if (line.IndexOf(ContentID) >= 0) // the line represent the distribution point of ContentID
                                        {
                                            // This DP string
                                            // The string always begins 27 characters into the line
                                            // This will return a final quote in the sting (which also needs stripping)
                                            SCCMDP = line.Substring(27, (line.IndexOf("',") - 27));
                                           
                                          
                                        }
                                    }
                                }
                            }
                            catch (Exception e)
                            { // if anything has gone wrong with the string it cant be fixed from here
                                LogError("Fatal Error attempting to read LocationServices.log: " + e.ToString(), "ERROR", 1);
                            }
                            finally
                            {
                                //reader.Close();
                                textReader.Close();
                                
                            }
                           // }
                           // else
                           // {
                           //     LogError("C:\\Windows\\CCM\\Logs\\LocationServices.log" + " Unreadable", "ERROR", 1);
                           //     ;
                           // }
                        }

                    }
                    catch (IOException e)
                    {
                        LogError("IO caught " + e.ToString(), "INFORMATION", 1);
                    }


                    if (SCCMDP.Length >= 0)
                    {

                        //This location needs to be written as a repair location for the application
                        RegistryKey regKey = Registry.ClassesRoot;
                        string newkey = @"Installer\Products\" + PackedGuid + @"\SourceList\URL";
                        // The successful install should have created all keys but the URL repair key
                        // Writing to classes root should dynamiclly determine where this point is
                        regKey.CreateSubKey(newkey,  RegistryKeyPermissionCheck.ReadWriteSubTree);

                        Registry.SetValue(@"HKEY_CLASSES_ROOT\Installer\Products\" + PackedGuid + @"\SourceList\URL", "1", SCCMDP, RegistryValueKind.ExpandString);

                        Console.WriteLine(@"RepairPoint: HKEY_CLASSES_ROOT\Installer\Products\" + PackedGuid + @"\SourceList\URL");
                        Console.WriteLine("SCCMDP = " + SCCMDP);
                    }
                }
            }
            else //sccm cache isnt in path
            {
                LogError("Installer external to SCCM cache " + path.IndexOf(SCCMCache), "INFORMATION", 1);
            }

            return;
        }



            public static string TraceFormat(string Text, string ErrorLevel, string ProcessID, string Component)
            {

                // Timezone offset
                DateTime dtNow = DateTime.Now;
                TimeZone tzCurrent = TimeZone.CurrentTimeZone;
                TimeSpan tsOffset = tzCurrent.GetUtcOffset(dtNow);

                // Get millseconds
                int ms = Math.Abs(dtNow.Millisecond);
                while (ms >= 100)
                    ms /= 100;

                // Format text
                return
                    "<![LOG[" + Text + "]LOG]!>"
                    + "<time=\"" + dtNow.Hour + ":" + dtNow.Minute + ":" + dtNow.Second + "." + ms.ToString() + "+" + tsOffset.TotalMinutes + "\""
                    + " date=\"" + dtNow.Month + "-" + dtNow.Day + "-" + dtNow.Year + "\""
                    + " component=\"" + Component + "\""
                    + " context=\"\""
                    + " type=\"" + ErrorLevel + "\""
                    + " thread=\"" + ProcessID + "\""
                    + " file=\"" + Component + "\">";

            }




            static class StringHelper
            {
                /// <summary>
                /// Receives string and returns the string with its letters reversed.
                /// </summary>
                public static string ReverseString(string s)
                {
                    char[] arr = s.ToCharArray();
                    Array.Reverse(arr);
                    return new string(arr);
                }
            }


            [DllImport("shell32.dll")]
            public static extern bool SHGetSpecialFolderPath(IntPtr hwndOwner, [Out]StringBuilder lpszPath, int nFolder, bool fCreate);

            static string GetSystemDirectory()
            {
                StringBuilder path = new StringBuilder(260);
                SHGetSpecialFolderPath(IntPtr.Zero,path,0x0029,false);
                return path.ToString();
              }






        //Convert String to UpperCase
        public static String Upper(String strParam)
        {
            return strParam.ToUpper();
        }

    }
}
