﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics; // used for running a program
using Microsoft.Win32; // Registry Reading
using System.IO;




/********************************************************
 * This Section deals with the physical installationa and removal of software
 * types.  
 * */



namespace InstallXML
{
    partial class Program
    {

        #region Executables
        public static string RunExe(string varEXEfile, string varSwitch, string varPackageName, string ReturnCodeString, string iName)
        {
            string ExeFullFath = "";
            
            //Parse the install exe path for variables such as comspec, windir etc
            //Console.WriteLine("Replace Variables");
            varEXEfile = ReplaceVariables(varEXEfile);
            //Console.WriteLine("Variables Finished");
            //We normally expect the executable to reside within the package folder
            //There are instances when executables must be run from ouside of the package directory
            //System32 is a good example.  If the executable is referenced from a drive mapping we wont
            //append the current path to the executable file

            varEXEfile = varEXEfile.Trim();

            // If a full file reference (either mapped driver or UNC doesnt exist
            // append the current directory
            if (@varEXEfile.IndexOf(":\\") == 1 || @varEXEfile.IndexOf(@"\\") == 0)
            {
                @ExeFullFath = @varEXEfile;
            }
            else
            {
                @ExeFullFath = path + (char)92 + @varEXEfile;
            }


          //  @ExeFullFath = (char)32 + @ExeFullFath + (char)32;

            ProcessStartInfo ps = new ProcessStartInfo(ExeFullFath);
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables(varSwitch);
            ps.WorkingDirectory = path;
            LogError(iName + "EXE: " + ExeFullFath + " ARGS = " + ps.Arguments, "INFORMATION", 3);
            //Console.WriteLine("EXE: " + ExeFullFath + " ARGS = " + ps.Arguments);
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();


            ReturnCode = process.ExitCode;
            //The exitcode 
            string[] exitcodes = ReturnCodeString.Split(',');

            bool installexitcode = false;

            foreach (string s in exitcodes)
            {
                Console.WriteLine("Allowable Exit Code = " + s);
                if (process.ExitCode.ToString() == s.Trim())
                {
                    //the received exit code is allowable
                    LogError(iName + " Allowable Exit Code Received = " + process.ExitCode, "INFORMATION", 3);
                    installexitcode = true;
                }

            }

           
            //Console.WriteLine("Actual Exit Code = " + process.ExitCode.ToString());

            if (installexitcode == true)
            {
                //Console.WriteLine("Successful Install");
                LogError(iName + " Successful Install ", "INFORMATION", 3);
                CreateStatusMIF(varPackageName, "SUCCESS", "Successful Install", iName);
                return "SUCCESS";
            }
            else
            {
                //Console.WriteLine("Failed Install");
                LogError(iName + " Invalid Exit Code Received = " + process.ExitCode, "ERROR", 3);
                ReturnCodeString = process.ExitCode.ToString();
                CreateStatusMIF(varPackageName, "FAIL", "Failed Install " + varEXEfile, iName);
                ReturnCode = process.ExitCode;
                return "FAIL";
            }

            

        }
        #endregion

        #region MSI 
        public static string RunMSI(string varMSIFile, string varMSTfile, string Switches, string varPackageName, string iName)
        {


            bool InstallRequired = true;
            
            string msiproductcode = MSIGetProductCode(varMSIFile);
            string msiproductversion = MSIGetProductVersion(varMSIFile);
            string msiversion = MSIGetProductVersion(varMSIFile);
            string[] MSIVersionNumbersString = msiversion.Split('.');

            //before the MSI is installed, a check for the existence of the product under HKCR should be performed
            //Console.WriteLine("PackedGuid =" + packguid(msiproductcode));
            RegistryKey masterKey = Registry.ClassesRoot.OpenSubKey("Installer\\Products\\" + packguid(msiproductcode));
            if (masterKey != null)
            {
                int[] InstalledVersionNumbers = versionfromreg(masterKey.GetValue("Version").ToString());
                string InstalledPackageName = masterKey.GetValue("ProductName").ToString();

                if (InstalledVersionNumbers.Length == 3)
                {
                    if (InstalledVersionNumbers[0] >= Convert.ToInt32(MSIVersionNumbersString[0])) //Major Version
                    {
                        if (InstalledVersionNumbers[1] >= Convert.ToInt32(MSIVersionNumbersString[1])) //Minor Version
                        {
                            if (InstalledVersionNumbers[2] >= Convert.ToInt32(MSIVersionNumbersString[2])) // Revision
                            {
                                //The installed product version is the same or higher than the version that is attempting to be installed
                                //We also need to ensure that the package we are installing is no a re-release of package
                                //We do this by comparing the Packagenames of what is installed compared to our package
                                if (InstalledVersionNumbers[2] == Convert.ToInt32(MSIVersionNumbersString[2])) // Revision
                                {
                                    if (InstalledPackageName != varPackageName)
                                    {
                                        //We have the same version of software but different packagename
                                        //this indicates the software is not an official package of is
                                        //a previous internal build
                                        LogError(iName + "InstalledPackageName = " + InstalledPackageName, "INFORMATION", 3);
                                        RunMSIRemoval(msiproductcode, InstalledPackageName,iName);
                                    }
                                    else
                                    {
                                        InstallRequired = false;
                                        Console.WriteLine("Product Already Installed");
                                    }
                                }

                            }
                            else
                            {
                                //The installed version is a build revision lower than than the MSI
                                //Remove the existing version
                                Console.WriteLine("Existing Product - lower version");
                                RunMSIRemoval(msiproductcode, InstalledPackageName, iName);
                            }
                        }
                        else
                        {
                            // the installed version is Minor version lower than the product installed
                            //Remove the existing version
                            Console.WriteLine("Existing Product - lower Minor Version");
                            RunMSIRemoval(msiproductcode, InstalledPackageName, iName);
                        }
                    }
                    else
                    {
                        //The installed version is a Major version lower than the product installed
                        //Remove the existing version
                        Console.WriteLine("Existing Product - lower Major Version");
                        RunMSIRemoval(msiproductcode, InstalledPackageName, iName);
                    }

                }
                masterKey.Close();
            }
            else
            {
                Console.WriteLine("Existing Product Not Found");
            }

            string msi_productcode = MSIGetProductCode(varMSIFile);


            if (InstallRequired == true)
            {
                // The commandline we run will be based upon the existence of a Transform
                if (varMSTfile == "") // We have no Transform
                {

                    ////if (DistributionPoint.Length > 0) // SOURCELIST Property must be appended
                    ////{
                    ////    //Console.WriteLine("Installing with Distribution Point " + DistributionPoint);
                    ////    LogError("Installing with Distribution Point " + DistributionPoint, "INFORMATION", 3);
                    ////    ProcessStartInfo ps = new ProcessStartInfo("Msiexec.exe");
                    ////    ps.UseShellExecute = false;
                    ////    ps.Verb = "runas";
                    ////    ps.Arguments = ReplaceVariables(" /i " +(char)34 + varMSIFile + (char)34 + " /qn ALLUSERS=1 REBOOT=ReallySuppress /l*v " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log" + (char)34 + " SOURCELIST=" + (char)34 + DistributionPoint + (char)34 + " " + Switches);
                    ////    ps.WorkingDirectory = path;
                    ////    Process process = System.Diagnostics.Process.Start(ps);
                    ////    process.WaitForExit();

                    ////    ReturnCode = process.ExitCode;

                    ////}
                    //else  // Install from current location
                    //{
                       // LogError("Installing without Distribution Point " + DistributionPoint, "INFORMATION", 3);
                        ProcessStartInfo ps = new ProcessStartInfo("Msiexec.exe");
                        ps.UseShellExecute = false;
                        ps.Verb = "runas";
                        ps.Arguments = ReplaceVariables(" /i " + (char)34 + varMSIFile + (char)34 + " /qn ALLUSERS=1 REBOOT=ReallySuppress /l*v " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log" + (char)34 + " " + Switches);
                        ps.WorkingDirectory = path;

                        Process process = System.Diagnostics.Process.Start(ps);
                        process.WaitForExit();

                        ReturnCode = process.ExitCode;

                  //  }

                }
                else // We do have a Transform to run
                {
                    LogError("Installing MSI With Transform", "INFORMATION", 3);
                    ////if (DistributionPoint.Length > 0) // SOURCELIST Property must be appended
                    ////{
                    ////    Console.WriteLine("Installing with Distribution Point " + DistributionPoint);
                    ////    ProcessStartInfo ps = new ProcessStartInfo("Msiexec.exe");
                    ////    ps.UseShellExecute = false;
                    ////    ps.Verb = "runas";
                    ////    ps.Arguments = ReplaceVariables(" /i " + (char)34 + varMSIFile + (char)34 + " /qn ALLUSERS=1 REBOOT=ReallySuppress TRANSFORMS=" + (char)34 + varMSTfile + (char)34 + " /l*v " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log" + (char)34 + " SOURCELIST=" + (char)34 + DistributionPoint + (char)34 + " " + Switches);
                    ////    ps.WorkingDirectory = path;
                    ////    Process process = System.Diagnostics.Process.Start(ps);
                    ////    process.WaitForExit();

                    ////    ReturnCode = process.ExitCode;
                    ////}
                    //else  // Install from current location
                    //{
                       // LogError("Installing MSI Without Distribution Point", "INFORMATION", 3);
                        ProcessStartInfo ps = new ProcessStartInfo("Msiexec.exe");
                        ps.UseShellExecute = false;
                        ps.Verb = "runas";
                        ps.Arguments = ReplaceVariables(" /i " + (char)34 + varMSIFile + (char)34 + " /qn ALLUSERS=1 REBOOT=ReallySuppress TRANSFORMS=" + (char)34 + varMSTfile + (char)34 + " /l*v " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log" + (char)34 + " " + Switches);
                        ps.WorkingDirectory = path;
                        Process process = System.Diagnostics.Process.Start(ps);
                        process.WaitForExit();
                        ReturnCode = process.ExitCode;

                    //}

                }

                // Wait for the Exit Code of Windows Installer - code will determine if install was successful
                LogError("MSIExec Return Code:" + ReturnCode, "INFORMATION", 3);
                //Evaluate Error Codes
                //All ErrorCodes apart from 3010 & 0 are seen as failures
                //Generate a Status MIF for Deployment System reporting
                switch (ReturnCode)
                {
                    case 0:
                        LogError("MSI Successful install " + ReturnCode, "INFORMATION", 3);

                        CheckCreateSCCM2012RepairPoint(packguid(msiproductcode));

                        CreateStatusMIF(varPackageName, "SUCCESS", "Successful Install", iName);
                        try
                        {
                            Console.WriteLine("Deleting Log: " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log" + (char)34);
                            System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log");
                            System.IO.File.WriteAllText(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".sig", Environment.UserName + " " + DateTime.Now);
                        }
                        catch
                        {
                        }
                        return "SUCCESS";
                    case 3010:
                        LogError("MSI Successful Install Reboot required" + ReturnCode, "INFORMATION", 3);
                        CheckCreateSCCM2012RepairPoint(packguid(msiproductcode));
                        CreateStatusMIF(varPackageName, "SUCCESS", "Successful Install - Reboot required", iName);
                        try
                        {
                            System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log");
                            System.IO.File.WriteAllText(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".sig", Environment.UserName + " " + DateTime.Now);
                        }
                        catch
                        {
                        }
                        return "SUCCESS";
                    default:
                        LogError("MSI Failure :" + ReturnCode, "INFORMATION", 3);
                        CreateStatusMIF(varPackageName, "FAIL", "ErrorCode: " + ReturnCode, iName);
                        return "FAIL";
                }
            }
            else // Install Not Required
            {
                LogError("MSI Success Product Already Installed" + ReturnCode, "INFORMATION", 3);
                CreateStatusMIF(varPackageName, "SUCCESS", "Product Already Installed", iName);
                return "SUCCESS";
            }



        }
        #endregion

        #region MSU
        public static string RunMSU(string varMSUFile, string iName)
        {
            Console.WriteLine((char)34 + varMSUFile + (char)34 + " /quiet /norestart /log:" + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + PackageName + "_MSU.log" + (char)34);
            ProcessStartInfo ps = new ProcessStartInfo("Wusa.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables((char)34 + varMSUFile + (char)34 + " /quiet /norestart /log:" + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + PackageName + "_MSU.log" + (char)34);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            ReturnCode = process.ExitCode;

            switch (process.ExitCode)
            {
                case 0:
                    CreateStatusMIF(PackageName, "SUCCESS", "Successful Patch install", iName);
                    ReturnCode = process.ExitCode;
                    return "SUCCESS";
                case 3010:
                    CreateStatusMIF(PackageName, "SUCCESS", "Successful Patch install - Reboot Required", iName);
                    ReturnCode = process.ExitCode;
                    return "SUCCESS";
                case 240006:
                    CreateStatusMIF(PackageName, "SUCCESS", "Successful - Patch already install", iName);
                    ReturnCode = process.ExitCode;
                    return "SUCCESS";
                case 2359302:
                    CreateStatusMIF(PackageName, "SUCCESS", "Successful - Patch already installed (2359302)", iName);
                    ReturnCode = process.ExitCode;
                    return "SUCCESS";
            }

            //If we haven't been returned, there is something horribly wrong.
            ReturnCode = process.ExitCode;
            return "FAILURE";
        }
        #endregion

        #region MSP
        public static string RunMSP(string varMSPFile, string iName)
        {

            //string fullpath = (char)34 + path + "\\" + varMSPFile + (char)34;

            ProcessStartInfo ps = new ProcessStartInfo("Msiexec.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables(" /p '" + varMSPFile + "' /qn REINSTALL=ALL REINSTALLMODE=omus /l*v " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + PackageName + "_MSIEXEC.log" + (char)34);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);

            Console.WriteLine("   Msiexec.exe " + ps.Arguments);

            process.WaitForExit();
            ReturnCode = process.ExitCode;

            //All ErrorCodes apart from 3101 1605 & 0 are seen as failures
            // Generate a Status MIF for Deployment System reporting

            switch (process.ExitCode)
            {
                case 0:
                    Console.WriteLine("  Return 0");
                    return "SUCCESS";
                case 3010:
                    Console.WriteLine("  Return 3010");
                    return "SUCCESS";
                case 1605:
                    Console.WriteLine("  Return 1605");
                    return "SUCCESS";
                default:
                    Console.WriteLine("  Return Fail " + process.ExitCode);
                    return "FAIL";
            }


        }
        #endregion

        #region APPV
        public static string RunAppV(string AppVManifest, string SftFile, string varPackageName, string iName)
        {
            ProcessStartInfo ps = new ProcessStartInfo("sftmime.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables("/ADD package:" + varPackageName + @" /manifest " + path + "\\" + AppVManifest + @" /overrideurl " + path + "\\" + SftFile + " /GLOBAL");
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            // Haven't even looked at APPV Exit Codes
            return "SUCCESS";
        }
        #endregion

        #region APPV Removal
        public static string RunAppVRM(string varPackageName, string iName)
        {

            ProcessStartInfo ps = new ProcessStartInfo("sftmime.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables(@"/UNPUBLISH package:" + varPackageName);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables(@"/DELETE package:" + varPackageName);
            ps.WorkingDirectory = path;
            //Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();
            return "SUCCESS";

        }
        #endregion

        #region MSI Removal
        public static string RunMSIRemoval(string varProductCode, string varPackageName, string iName)
        {
            // Function purpose is to remove MSI Productcodes
            // ReturnCodes are checked to ensure that the removal is successful
            ProcessStartInfo ps = new ProcessStartInfo("Msiexec.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables(" /x " + varProductCode + " /qn REBOOT=ReallySuppress /l*v " + (char)34 + System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log" + (char)34);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            ReturnCode = process.ExitCode;

            Console.WriteLine();
            Console.ForegroundColor = ConsoleColor.Cyan;
            Console.WriteLine("Item: " + iName + "MSIExec Return Code:" + process.ExitCode);
            Console.ForegroundColor = ConsoleColor.White;

            //Evaluate Error Codes
            //All ErrorCodes apart from 3010 1605 & 0 are seen as failures

            switch (process.ExitCode)
            {
                case 0:
                    Console.WriteLine("{0}", "Successful Removal ");
                    CreateStatusMIF(varPackageName, "SUCCESS", "Successful Removal", iName);
                    try
                    {
                        System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log");
                        System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".sig");
                    }
                    catch
                    {
                    }
                    return "SUCCESS";
                case 3010:
                    Console.WriteLine("{0}", "Successful Removal Reboot required");
                    CreateStatusMIF(varPackageName, "SUCCESS", "Successful Removal - Reboot Required", iName);
                    try
                    {
                        System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log");
                        System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + ".sig");
                    }
                    catch
                    {
                    }
                    return "SUCCESS";
                case 1605:
                    Console.WriteLine("{0}", "Successful Removal - Product Not Present");
                    CreateStatusMIF(varPackageName, "SUCCESS", "Successful Removal - Product Not Present", iName);
                    try
                    {
                        System.IO.File.Delete(System.Environment.ExpandEnvironmentVariables("%WinDir%") + "\\Logs\\" + varPackageName + "_MSIEXEC.log");
                    }
                    catch
                    {
                    }
                    return "SUCCESS";
                default:
                    Console.WriteLine("{0}", "Error " + process.ExitCode);
                    return "FAIL";
            }


        }
        #endregion

        #region VBScript
        /// <summary>
        /// 
        /// </summary>
        /// <param name="varVBSFile"></param>
        /// <param name="varSwitch"></param>
        /// <param name="iArgs2"></param>
        /// <param name="varPackageName"></param>
        /// <param name="iName"></param>
        /// <returns></returns>
        public static string RunVBS(string varVBSFile, string varSwitch, string iArgs2, string varPackageName, string iName)
        {
          
            ProcessStartInfo ps = new ProcessStartInfo("cscript.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables((char)34 + CheckReturnFullPath(varVBSFile) + (char)34 + " " + varSwitch);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            ReturnCode = process.ExitCode;

            //The exitcode 
            string[] exitcodes = iArgs2.Split(',');
            bool installexitcode = false;
            foreach (string s in exitcodes)
            {
              //  Console.WriteLine("Allowable Exit Code = " + s);
                if (process.ExitCode.ToString() == s.Trim())
                {
                    //the received exit code is allowable
                    installexitcode = true;
                }
            }

            Console.WriteLine("Actual Exit Code = " + process.ExitCode.ToString());

            if (installexitcode == true)
            {
                Console.WriteLine("{0}", "Successful Install");
                CreateStatusMIF(varPackageName, "SUCCESS", "Successful Install", iName);
                return "SUCCESS";
            }
            else
            {
                Console.WriteLine("{0}", "Error - Exit Code: " + process.ExitCode);
                ReturnCode = process.ExitCode;
                CreateStatusMIF(varPackageName, "FAIL", "Failed Install " + process.ExitCode, iName);
                return "FAIL " + process.ExitCode;
            }


        }
        #endregion


        #region CMD Batch File
        public static string RunCMD(string varCMDFile, string varSwitch, string varPackageName, string iName)
        {
            ProcessStartInfo ps = new ProcessStartInfo("Cmd.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables(@"/C " + (char)34 + CheckReturnFullPath(varCMDFile) + (char)34 + " " + varSwitch);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            ReturnCode = process.ExitCode;

            //ReturnCode = process.ExitCode;
            switch (process.ExitCode)
            {
                case 0:
                    Console.WriteLine("{0}", "Successful Install");
                    CreateStatusMIF(varPackageName, "SUCCESS", "Successful Install", iName);
                    return "SUCCESS";
                default:
                    // If the errorcode is not 0 we assume an Error
                    Console.WriteLine("{0}", "Error - Exit Code: " + process.ExitCode);
                    return "FAIL";
            }

        }
        #endregion


        #region Powershell File
        public static string RunPS1(string varPSFile, string varSwitch, string ReturnCodeString, string iName)
        {
            ProcessStartInfo ps = new ProcessStartInfo(@"powershell.exe");
            ps.UseShellExecute = false;
            ps.Verb = "runas";
            ps.Arguments = ReplaceVariables("-File " + (char)34 + CheckReturnFullPath(varPSFile) + (char)34 + " " + varSwitch);
            ps.WorkingDirectory = path;
            Process process = System.Diagnostics.Process.Start(ps);
            process.WaitForExit();

            ReturnCode = process.ExitCode;
            //The exitcode - ensure that '0' is an allowed exit code

            string[] exitcodes = ReturnCodeString.Split(',');

            bool installexitcode = false;

            foreach (string s in exitcodes)
            {
                Console.WriteLine("Powershell Allowable Exit Code = " + s);
                if (process.ExitCode.ToString() == s.Trim() ||  process.ExitCode.ToString() == "0")
                {
                    //the received exit code is allowable
                    LogError(iName + " Powershell Allowable Exit Code Received = " + process.ExitCode, "INFORMATION", 3);
                    installexitcode = true;
                }

            }



            if (installexitcode == true)
            {
                //Console.WriteLine("Successful Install");
                LogError(iName + "Powershell Install Success" + " Return Code = " + ReturnCode, "INFORMATION", 3);
                CreateStatusMIF(varPSFile, "SUCCESS", "Successful Install", iName);
                return "SUCCESS";
            }
            else
            {
                //Console.WriteLine("Failed Install");
                LogError(iName + "Powershell Install Failure" + " Return Code = " + ReturnCode, "ERROR", 3);
                ReturnCode = process.ExitCode;
                return "FAIL";
            }

        }
        #endregion



    }
}
